define(["exports", "./vaadin-dir-helper.js"], function (_exports, _vaadinDirHelper) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.DirMixin = void 0;

  function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = babelHelpers.getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = babelHelpers.getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return babelHelpers.possibleConstructorReturn(this, result); }; }

  function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Date.prototype.toString.call(Reflect.construct(Date, [], function () {})); return true; } catch (e) { return false; } }

  /**
   * Array of Vaadin custom element classes that have been subscribed to the dir changes.
   */
  var directionSubscribers = [];

  var directionUpdater = function directionUpdater() {
    var documentDir = getDocumentDir();
    directionSubscribers.forEach(function (element) {
      alignDirs(element, documentDir);
    });
  };

  var scrollType;
  var directionObserver = new MutationObserver(directionUpdater);
  directionObserver.observe(document.documentElement, {
    attributes: true,
    attributeFilter: ['dir']
  });

  var alignDirs = function alignDirs(element, documentDir) {
    if (documentDir) {
      element.setAttribute('dir', documentDir);
    } else {
      element.removeAttribute('dir');
    }
  };

  var getDocumentDir = function getDocumentDir() {
    return document.documentElement.getAttribute('dir');
  };
  /**
   * @polymerMixin
   */


  var DirMixin = function DirMixin(superClass) {
    return /*#__PURE__*/function (_superClass) {
      babelHelpers.inherits(VaadinDirMixin, _superClass);

      var _super = _createSuper(VaadinDirMixin);

      function VaadinDirMixin() {
        babelHelpers.classCallCheck(this, VaadinDirMixin);
        return _super.apply(this, arguments);
      }

      babelHelpers.createClass(VaadinDirMixin, [{
        key: "connectedCallback",

        /** @protected */
        value: function connectedCallback() {
          babelHelpers.get(babelHelpers.getPrototypeOf(VaadinDirMixin.prototype), "connectedCallback", this).call(this);

          if (!this.hasAttribute('dir')) {
            this.__subscribe();

            alignDirs(this, getDocumentDir());
          }
        }
        /** @protected */

      }, {
        key: "attributeChangedCallback",
        value: function attributeChangedCallback(name, oldValue, newValue) {
          babelHelpers.get(babelHelpers.getPrototypeOf(VaadinDirMixin.prototype), "attributeChangedCallback", this).call(this, name, oldValue, newValue);

          if (name !== 'dir') {
            return;
          } // New value equals to the document direction and the element is not subscribed to the changes


          var newValueEqlDocDir = newValue === getDocumentDir() && directionSubscribers.indexOf(this) === -1; // Value was emptied and the element is not subscribed to the changes

          var newValueEmptied = !newValue && oldValue && directionSubscribers.indexOf(this) === -1; // New value is different and the old equals to document direction and the element is not subscribed to the changes

          var newDiffValue = newValue !== getDocumentDir() && oldValue === getDocumentDir();

          if (newValueEqlDocDir || newValueEmptied) {
            this.__subscribe();

            alignDirs(this, getDocumentDir());
          } else if (newDiffValue) {
            this.__subscribe(false);
          }
        }
        /** @protected */

      }, {
        key: "disconnectedCallback",
        value: function disconnectedCallback() {
          babelHelpers.get(babelHelpers.getPrototypeOf(VaadinDirMixin.prototype), "disconnectedCallback", this).call(this);

          this.__subscribe(false);

          this.removeAttribute('dir');
        }
        /** @private */

      }, {
        key: "__subscribe",
        value: function __subscribe() {
          var push = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

          if (push) {
            directionSubscribers.indexOf(this) === -1 && directionSubscribers.push(this);
          } else {
            directionSubscribers.indexOf(this) > -1 && directionSubscribers.splice(directionSubscribers.indexOf(this), 1);
          }
        }
        /**
         * @param {Element} element
         * @return {number}
         * @protected
         */

      }, {
        key: "__getNormalizedScrollLeft",
        value: function __getNormalizedScrollLeft(element) {
          return _vaadinDirHelper.DirHelper.getNormalizedScrollLeft(scrollType, this.getAttribute('dir') || 'ltr', element);
        }
        /**
         * @param {Element} element
         * @param {number} scrollLeft
         * @protected
         */

      }, {
        key: "__setNormalizedScrollLeft",
        value: function __setNormalizedScrollLeft(element, scrollLeft) {
          return _vaadinDirHelper.DirHelper.setNormalizedScrollLeft(scrollType, this.getAttribute('dir') || 'ltr', element, scrollLeft);
        }
      }], [{
        key: "finalize",

        /** @protected */
        value: function finalize() {
          babelHelpers.get(babelHelpers.getPrototypeOf(VaadinDirMixin), "finalize", this).call(this);

          if (!scrollType) {
            scrollType = _vaadinDirHelper.DirHelper.detectScrollType();
          }
        }
      }, {
        key: "properties",
        get: function get() {
          return {
            /**
             * @protected
             */
            dir: {
              type: String,
              readOnly: true
            }
          };
        }
      }]);
      return VaadinDirMixin;
    }(superClass);
  };

  _exports.DirMixin = DirMixin;
});